@@ -5,30 +5,33 @@
 """
 Build and index fields within documents.
 """
+import math
+
 from collections import namedtuple
 from datetime import datetime, date
 from decimal import Decimal
+from typing import Any, Callable, Tuple, Union
 
-from dateutil import tz
 from psycopg2.extras import NumericRange, DateTimeTZRange
 from sqlalchemy import cast, func, and_
 from sqlalchemy.dialects import postgresql as postgres
-from sqlalchemy.dialects.postgresql import INT4RANGE
 from sqlalchemy.dialects.postgresql import NUMRANGE, TSTZRANGE
 from sqlalchemy.sql import ColumnElement
+from sqlalchemy.orm import aliased
 
 from datacube import utils
 from datacube.model.fields import Expression, Field
 from datacube.model import Range
 from datacube.utils import get_doc_offset_safe
-from .sql import FLOAT8RANGE
 
-from typing import Any, Callable, Tuple, Union
+from datacube.drivers.postgis._schema import Dataset, search_field_index_map
+from datacube.utils import cached_property
+from datacube.utils.dates import tz_aware
 
 
 class PgField(Field):
     """
-    Postgres implementation of a searchable field. May be a value inside
+    Postgis implementation of a searchable field. May be a value inside
     a JSONB column.
     """
 
@@ -40,9 +43,30 @@ class PgField(Field):
         self.indexed = indexed
 
     @property
-    def required_alchemy_table(self):
+    def select_alchemy_table(self):
         return self.alchemy_column.table
 
+    @cached_property
+    def search_index_table(self):
+        if self.indexed:
+            search_table = search_field_index_map[self.type_name]
+            return aliased(search_table, name=f"{search_table.__tablename__}-{self.name}")
+        else:
+            return self.select_alchemy_table
+
+    @cached_property
+    def dataset_join_args(self):
+        if self.indexed:
+            return (
+                self.search_index_table,
+                and_(
+                    Dataset.id == self.search_index_table.dataset_ref,
+                    self.search_index_table.search_key == self.name
+                )
+            )
+        else:
+            return (self.search_index_table,)
+
     @property
     def alchemy_expression(self):
         """
@@ -51,6 +75,13 @@ class PgField(Field):
         """
         raise NotImplementedError('alchemy expression')
 
+    @property
+    def search_alchemy_expression(self):
+        if self.indexed:
+            return self.search_index_table.search_val
+        else:
+            return self.alchemy_expression
+
     @property
     def sql_expression(self):
         """
@@ -62,10 +93,6 @@ class PgField(Field):
             compile_kwargs={"literal_binds": True}
         ))
 
-    @property
-    def postgres_index_type(self):
-        return 'btree'
-
     def __eq__(self, value):
         """
         :rtype: Expression
@@ -96,17 +123,11 @@ class NativeField(PgField):
         expression = self._expression if self._expression is not None else self.alchemy_column
         return expression.label(self.name)
 
-    @property
-    def postgres_index_type(self):
-        # Don't add extra indexes for native fields.
-        return None
-
 
 class PgDocField(PgField):
     """
     A field extracted from inside a (jsonb) document.
     """
-
     def extract(self, document):
         """
         Extract a value from the given document in pure python (no postgres).
@@ -122,14 +143,24 @@ class PgDocField(PgField):
         # Default do nothing (eg. string datatypes)
         return value
 
+    def search_value_to_alchemy(self, value):
+        """
+        Value to use in search tables.  Identical to value_to_alchemy, unless it needs to be promoted to range type.
+
+        :param value:
+        :return:
+        """
+        return self.value_to_alchemy(value)
+
     def parse_value(self, value):
         """
         Parse the value from a string. May be overridden by subclasses.
         """
         return value
 
-    def _alchemy_offset_value(self, doc_offsets, agg_function):
-        # type: (Tuple[Tuple[str]], Callable[[Any], ColumnElement]) -> ColumnElement
+    def _alchemy_offset_value(self,
+                              doc_offsets: Tuple[Tuple[str]],
+                              agg_function: Callable[[Any], ColumnElement]) -> ColumnElement:
         """
         Get an sqlalchemy value for the given offsets of this field's sqlalchemy column.
         If there are multiple they will be combined using the given aggregate function.
@@ -213,17 +244,8 @@ class SimpleDocField(PgDocField):
         return self.extract(ctx)
 
 
-class IntDocField(SimpleDocField):
-    type_name = 'integer'
-
-    def value_to_alchemy(self, value):
-        return cast(value, postgres.INTEGER)
-
-    def between(self, low, high):
-        return ValueBetweenExpression(self, low, high)
-
-    def parse_value(self, value):
-        return int(value)
+class UnindexableValue(Exception):
+    pass
 
 
 class NumericDocField(SimpleDocField):
@@ -232,6 +254,17 @@ class NumericDocField(SimpleDocField):
     def value_to_alchemy(self, value):
         return cast(value, postgres.NUMERIC)
 
+    def search_value_to_alchemy(self, value):
+        if isinstance(value, float) and math.isnan(value):
+            raise UnindexableValue("Cannot index NaNs")
+        alc_val = self.value_to_alchemy(value)
+        return func.numrange(
+            alc_val, alc_val,
+            # Inclusive on both sides.
+            '[]',
+            type_=NUMRANGE,
+        )
+
     def between(self, low, high):
         return ValueBetweenExpression(self, low, high)
 
@@ -239,34 +272,45 @@ class NumericDocField(SimpleDocField):
         return Decimal(value)
 
 
-class DoubleDocField(SimpleDocField):
+class IntDocField(NumericDocField):
+    type_name = 'integer'
+
+    def parse_value(self, value):
+        return int(value)
+
+
+class DoubleDocField(NumericDocField):
     type_name = 'double'
 
-    def value_to_alchemy(self, value):
-        return cast(value, postgres.DOUBLE_PRECISION)
-
-    def between(self, low, high):
-        return ValueBetweenExpression(self, low, high)
-
     def parse_value(self, value):
         return float(value)
 
 
+DateFieldLike = Union[datetime, date, str, ColumnElement]
+
+
 class DateDocField(SimpleDocField):
     type_name = 'datetime'
 
-    def value_to_alchemy(self, value):
-        # type: (Union[datetime, date, str, ColumnElement]) -> Union[datetime, date, str, ColumnElement]
+    def value_to_alchemy(self, value: DateFieldLike) -> DateFieldLike:
         """
         Wrap a value as needed for this field type.
         """
         if isinstance(value, datetime):
-            return _default_utc(value)
+            return tz_aware(value)
         # SQLAlchemy expression or string are parsed in pg as dates.
         elif isinstance(value, (ColumnElement, str)):
             return func.odc.common_timestamp(value)
         else:
-            raise ValueError("Value not readable as date: %r" % (value,))
+            raise ValueError("Value not readable as date: %r" % value)
+
+    def search_value_to_alchemy(self, value):
+        return func.tstzrange(
+            value, value,
+            # Inclusive on both sides.
+            '[]',
+            type_=TSTZRANGE,
+        )
 
     def between(self, low, high):
         return ValueBetweenExpression(self, low, high)
@@ -314,10 +358,6 @@ class RangeDocField(PgDocField):
     def value_to_alchemy(self, value):
         raise NotImplementedError('range type')
 
-    @property
-    def postgres_index_type(self):
-        return 'gist'
-
     @property
     def alchemy_expression(self):
         return self.value_to_alchemy((self.lower.alchemy_expression, self.greater.alchemy_expression))
@@ -358,45 +398,15 @@ class NumericRangeDocField(RangeDocField):
         return RangeBetweenExpression(self, low, high, _range_class=NumericRange)
 
 
-class IntRangeDocField(RangeDocField):
+class IntRangeDocField(NumericRangeDocField):
     FIELD_CLASS = IntDocField
     type_name = 'integer-range'
 
-    def value_to_alchemy(self, value):
-        low, high = value
-        return func.numrange(
-            low, high,
-            # Inclusive on both sides.
-            '[]',
-            type_=INT4RANGE,
-        )
 
-    def between(self, low, high):
-        """
-        :rtype: Expression
-        """
-        return RangeBetweenExpression(self, low, high, _range_class=NumericRange)
-
-
-class DoubleRangeDocField(RangeDocField):
+class DoubleRangeDocField(NumericRangeDocField):
     FIELD_CLASS = DoubleDocField
     type_name = 'double-range'
 
-    def value_to_alchemy(self, value):
-        low, high = value
-        return func.odc.float8range(
-            low, high,
-            # Inclusive on both sides.
-            '[]',
-            type_=FLOAT8RANGE,
-        )
-
-    def between(self, low, high):
-        """
-        :rtype: Expression
-        """
-        return RangeBetweenExpression(self, low, high, _range_class=NumericRange)
-
 
 class DateRangeDocField(RangeDocField):
     FIELD_CLASS = DateDocField
@@ -421,8 +431,8 @@ class DateRangeDocField(RangeDocField):
         if isinstance(low, datetime) and isinstance(high, datetime):
             return RangeBetweenExpression(
                 self,
-                _default_utc(low),
-                _default_utc(high),
+                tz_aware(low),
+                tz_aware(high),
                 _range_class=DateTimeTZRange
             )
         else:
@@ -430,8 +440,7 @@ class DateRangeDocField(RangeDocField):
                              "expecting datetimes, got: (%r, %r)" % (low, high))
 
 
-def _number_implies_year(v):
-    # type: (Union[int, datetime]) -> datetime
+def _number_implies_year(v: Union[int, datetime]) -> datetime:
     """
     >>> _number_implies_year(1994)
     datetime.datetime(1994, 1, 1, 0, 0)
@@ -471,13 +480,12 @@ class ValueBetweenExpression(PgExpression):
     @property
     def alchemy_expression(self):
         if self.low_value is not None and self.high_value is not None:
-            return and_(self.field.alchemy_expression >= self.low_value,
-                        self.field.alchemy_expression < self.high_value)
+            return and_(self.field.search_alchemy_expression >= self.low_value,
+                        self.field.search_alchemy_expression <= self.high_value)
         if self.low_value is not None:
-            return self.field.alchemy_expression >= self.low_value
+            return self.field.search_alchemy_expression >= self.low_value
         if self.high_value is not None:
-            return self.field.alchemy_expression < self.high_value
-
+            return self.field.search_alchemy_expression <= self.high_value
         raise ValueError('Expect at least one of [low,high] to be set')
 
 
@@ -487,12 +495,11 @@ class RangeBetweenExpression(PgExpression):
         self.low_value = low_value
         self.high_value = high_value
         self._range_class = _range_class
+        self._alc_val = self._range_class(self.low_value, self.high_value, bounds='[]')
 
     @property
     def alchemy_expression(self):
-        return self.field.alchemy_expression.overlaps(
-            self._range_class(self.low_value, self.high_value, bounds='[]')
-        )
+        return self.field.search_alchemy_expression.overlaps(self._alc_val)
 
 
 class RangeContainsExpression(PgExpression):
@@ -502,7 +509,7 @@ class RangeContainsExpression(PgExpression):
 
     @property
     def alchemy_expression(self):
-        return self.field.alchemy_expression.contains(self.value)
+        return self.field.search_alchemy_expression.contains(self.value)
 
 
 class EqualsExpression(PgExpression):
@@ -512,7 +519,7 @@ class EqualsExpression(PgExpression):
 
     @property
     def alchemy_expression(self):
-        return self.field.alchemy_expression == self.value
+        return self.field.search_alchemy_expression == self.value
 
     def evaluate(self, ctx):
         return self.field.evaluate(ctx) == self.value
@@ -614,12 +621,6 @@ def _coalesce(*values):
     return None
 
 
-def _default_utc(d):
-    if d.tzinfo is None:
-        return d.replace(tzinfo=tz.tzutc())
-    return d
-
-
 # How to choose/combine multiple doc values.
 ValueAggregation = namedtuple('ValueAggregation', ('calc', 'pg_calc'))
 SELECTION_TYPES = {
