@@ -37,7 +37,7 @@ _LOG = logging.getLogger(__name__)
 try:
     import pwd
 
-    DEFAULT_DB_USER = pwd.getpwuid(os.geteuid()).pw_name  # type: Optional[str]
+    DEFAULT_DB_USER: Optional[str] = pwd.getpwuid(os.geteuid()).pw_name
 except (ImportError, KeyError):
     # No default on Windows and some other systems
     DEFAULT_DB_USER = None
@@ -225,6 +225,7 @@ class PostgresDb(object):
         """
         with self._engine.connect() as connection:
             try:
+                connection.execution_options(isolation_level="AUTOCOMMIT")
                 yield _api.PostgresDbAPI(connection)
             finally:
                 connection.close()
