@@ -8,8 +8,8 @@ Build and index fields within documents.
 from collections import namedtuple
 from datetime import datetime, date
 from decimal import Decimal
+from typing import Any, Callable, Tuple, Union
 
-from dateutil import tz
 from psycopg2.extras import NumericRange, DateTimeTZRange
 from sqlalchemy import cast, func, and_
 from sqlalchemy.dialects import postgresql as postgres
@@ -23,7 +23,7 @@ from datacube.model import Range
 from datacube.utils import get_doc_offset_safe
 from .sql import FLOAT8RANGE
 
-from typing import Any, Callable, Tuple, Union
+from datacube.utils.dates import tz_aware
 
 
 class PgField(Field):
@@ -128,8 +128,9 @@ class PgDocField(PgField):
         """
         return value
 
-    def _alchemy_offset_value(self, doc_offsets, agg_function):
-        # type: (Tuple[Tuple[str]], Callable[[Any], ColumnElement]) -> ColumnElement
+    def _alchemy_offset_value(self,
+                              doc_offsets: Tuple[Tuple[str]],
+                              agg_function: Callable[[Any], ColumnElement]) -> ColumnElement:
         """
         Get an sqlalchemy value for the given offsets of this field's sqlalchemy column.
         If there are multiple they will be combined using the given aggregate function.
@@ -255,13 +256,14 @@ class DoubleDocField(SimpleDocField):
 class DateDocField(SimpleDocField):
     type_name = 'datetime'
 
-    def value_to_alchemy(self, value):
-        # type: (Union[datetime, date, str, ColumnElement]) -> Union[datetime, date, str, ColumnElement]
+    def value_to_alchemy(self,
+                         value: Union[datetime, date, str, ColumnElement]
+                        ) -> Union[datetime, date, str, ColumnElement]:
         """
         Wrap a value as needed for this field type.
         """
         if isinstance(value, datetime):
-            return _default_utc(value)
+            return tz_aware(value)
         # SQLAlchemy expression or string are parsed in pg as dates.
         elif isinstance(value, (ColumnElement, str)):
             return func.agdc.common_timestamp(value)
@@ -421,8 +423,8 @@ class DateRangeDocField(RangeDocField):
         if isinstance(low, datetime) and isinstance(high, datetime):
             return RangeBetweenExpression(
                 self,
-                _default_utc(low),
-                _default_utc(high),
+                tz_aware(low),
+                tz_aware(high),
                 _range_class=DateTimeTZRange
             )
         else:
@@ -430,8 +432,7 @@ class DateRangeDocField(RangeDocField):
                              "expecting datetimes, got: (%r, %r)" % (low, high))
 
 
-def _number_implies_year(v):
-    # type: (Union[int, datetime]) -> datetime
+def _number_implies_year(v: Union[int, datetime]) -> datetime:
     """
     >>> _number_implies_year(1994)
     datetime.datetime(1994, 1, 1, 0, 0)
@@ -614,12 +615,6 @@ def _coalesce(*values):
     return None
 
 
-def _default_utc(d):
-    if d.tzinfo is None:
-        return d.replace(tzinfo=tz.tzutc())
-    return d
-
-
 # How to choose/combine multiple doc values.
 ValueAggregation = namedtuple('ValueAggregation', ('calc', 'pg_calc'))
 SELECTION_TYPES = {
