@@ -2,6 +2,8 @@
 #
 # Copyright (c) 2015-2020 ODC Contributors
 # SPDX-License-Identifier: Apache-2.0
+
+
 class DuplicateRecordError(Exception):
     pass
 
@@ -12,3 +14,9 @@ class MissingRecordError(Exception):
 
 class IndexSetupError(Exception):
     pass
+
+
+class TransactionException(Exception):  # noqa: N818
+    def __init__(self, *args, commit=False, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.commit = commit
