@@ -9,21 +9,23 @@ import json
 import logging
 import warnings
 from collections import namedtuple
-from typing import Iterable, Union, List
+from time import monotonic
+from typing import Iterable, List, Union, Mapping, Any
 from uuid import UUID
 
 from sqlalchemy import select, func
 
 from datacube.drivers.postgres._fields import SimpleDocField, DateDocField
 from datacube.drivers.postgres._schema import DATASET
-from datacube.index.abstract import AbstractDatasetResource, DatasetSpatialMixin, DSID
+from datacube.index.abstract import AbstractDatasetResource, DatasetSpatialMixin, DSID, DatasetTuple, BatchStatus
 from datacube.index.postgres._transaction import IndexResourceAddIn
-from datacube.model import Dataset, DatasetType
+from datacube.model import Dataset, Product
 from datacube.model.fields import Field
 from datacube.model.utils import flatten_datasets
 from datacube.utils import jsonify_document, _readable_offset, changes
 from datacube.utils.changes import get_doc_changes
 from datacube.index import fields
+from datacube.drivers.postgres._api import split_uri
 
 _LOG = logging.getLogger(__name__)
 
@@ -42,8 +44,7 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
         :type dataset_type_resource: datacube.index._products.ProductResource
         """
         self._db = db
-        self._index = index
-        self.types = self._index.products
+        super().__init__(index)
 
     def get(self, id_: Union[str, UUID], include_sources=False):
         """
@@ -150,7 +151,7 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
 
             # First insert all new datasets
             for ds in dss:
-                is_new = transaction.insert_dataset(ds.metadata_doc_without_lineage(), ds.id, ds.type.id)
+                is_new = transaction.insert_dataset(ds.metadata_doc_without_lineage(), ds.id, ds.product.id)
                 sources = ds.sources
                 if is_new and sources is not None:
                     edges.extend((name, ds.id, src.id)
@@ -189,7 +190,39 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
 
         return dataset
 
-    def search_product_duplicates(self, product: DatasetType, *args):
+    def _add_batch(self, batch_ds: Iterable[DatasetTuple], cache: Mapping[str, Any]) -> BatchStatus:
+        b_started = monotonic()
+        batch = {
+            "datasets": [],
+            "uris": [],
+        }
+        for prod, metadata_doc, uris in batch_ds:
+            dsid = UUID(metadata_doc["id"])
+            batch["datasets"].append(
+                {
+                    "id": dsid,
+                    "dataset_type_ref": prod.id,
+                    "metadata": metadata_doc,
+                    "metadata_type_ref": prod.metadata_type.id
+                }
+            )
+            for uri in uris:
+                scheme, body = split_uri(uri)
+                batch["uris"].append(
+                    {
+                        "dataset_ref": dsid,
+                        "uri_scheme": scheme,
+                        "uri_body": body,
+                    }
+                )
+        with self._db_connection(transaction=True) as connection:
+            if batch["datasets"]:
+                b_added, b_skipped = connection.insert_dataset_bulk(batch["datasets"])
+            if batch["uris"]:
+                connection.insert_dataset_location_bulk(batch["uris"])
+        return BatchStatus(b_added, b_skipped, monotonic() - b_started)
+
+    def search_product_duplicates(self, product: Product, *args):
         """
         Find dataset ids who have duplicates of the given set of field names.
 
@@ -204,7 +237,7 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
             assert isinstance(f, fields.Field), "Not a field: %r" % (f,)
             return f
 
-        group_fields = [load_field(f) for f in args]  # type: List[fields.Field]
+        group_fields: List[fields.Field] = [load_field(f) for f in args]
         result_type = namedtuple('search_result', list(f.name for f in group_fields))  # type: ignore
 
         expressions = [product.metadata_type.dataset_fields.get('product') == product.name]
@@ -228,9 +261,9 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
         if not existing:
             raise ValueError('Unknown dataset %s, cannot update – did you intend to add it?' % dataset.id)
 
-        if dataset.type.name != existing.type.name:
-            raise ValueError('Changing product is not supported. From %s to %s in %s' % (existing.type.name,
-                                                                                         dataset.type.name,
+        if dataset.product.name != existing.product.name:
+            raise ValueError('Changing product is not supported. From %s to %s in %s' % (existing.product.name,
+                                                                                         dataset.product.name,
                                                                                          dataset.id))
 
         # TODO: figure out (un)safe changes from metadata type?
@@ -276,7 +309,7 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
 
         _LOG.info("Updating dataset %s", dataset.id)
 
-        product = self.types.get_by_name(dataset.type.name)
+        product = self.types.get_by_name(dataset.product.name)
         with self._db_connection(transaction=True) as transaction:
             if not transaction.update_dataset(dataset.metadata_doc_without_lineage(), dataset.id, product.id):
                 raise ValueError("Failed to update dataset %s..." % dataset.id)
@@ -467,7 +500,7 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
         product = product or self.types.get(dataset_res.dataset_type_ref)
 
         return Dataset(
-            type_=product,
+            product=product,
             metadata_doc=dataset_res.metadata,
             uris=uris,
             indexed_by=dataset_res.added_by if full_info else None,
@@ -513,7 +546,7 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
         Perform a search, returning datasets grouped by product type.
 
         :param dict[str,str|float|datacube.model.Range] query:
-        :rtype: __generator[(DatasetType,  __generator[Dataset])]]
+        :rtype: __generator[(Product,  __generator[Dataset])]]
         """
         for product, datasets in self._do_search_by_product(query):
             yield product, self._make_many(datasets, product)
@@ -561,7 +594,7 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
 
         :param dict[str,str|float|datacube.model.Range] query:
         :returns: Sequence of (product, count)
-        :rtype: __generator[(DatasetType,  int)]]
+        :rtype: __generator[(Product,  int)]]
         """
         return self._do_count_by_product(query)
 
@@ -573,7 +606,7 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
         :param dict[str,str|float|datacube.model.Range] query:
         :param str period: Time range for each slice: '1 month', '1 day' etc.
         :returns: For each matching product type, a list of time ranges and their count.
-        :rtype: __generator[(DatasetType, list[(datetime.datetime, datetime.datetime), int)]]
+        :rtype: __generator[(Product, list[(datetime.datetime, datetime.datetime), int)]]
         """
         return self._do_time_count(period, query)
 
@@ -886,3 +919,10 @@ class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
 
     def spatial_extent(self, ids, crs=None):
         return None
+
+    def get_all_docs_for_product(self, product: Product, batch_size: int = 1000) -> Iterable[DatasetTuple]:
+        product_search_key = [product.name]
+        with self._db_connection(transaction=True) as connection:
+            for row in connection.bulk_simple_dataset_search(products=product_search_key, batch_size=batch_size):
+                prod_name, metadata_doc, uris = tuple(row)
+                yield DatasetTuple(product, metadata_doc, uris)
