@@ -16,7 +16,7 @@ import click
 from datacube import config, __version__
 from datacube.api.core import Datacube
 
-from datacube.executor import get_executor, mk_celery_executor  # type: ignore[attr-defined]
+from datacube.executor import get_executor  # type: ignore[attr-defined]
 from datacube.index import index_connect
 
 from datacube.ui.expression import parse_expressions
@@ -77,7 +77,7 @@ class ClickHandler(logging.Handler):
         try:
             msg = self.format(record)
             click.echo(msg, err=True)
-        except:  # noqa: E772  pylint: disable=bare-except
+        except:   # pylint: disable=bare-except  # noqa: E722
             self.handleError(record)
 
 
@@ -270,7 +270,6 @@ EXECUTOR_TYPES = {
     'serial': lambda _: get_executor(None, None),
     'multiproc': lambda workers: get_executor(None, int(workers)),
     'distributed': lambda addr: get_executor(addr, True),
-    'celery': lambda addr: mk_celery_executor(*parse_endpoint(addr))
 }
 
 EXECUTOR_TYPES['dask'] = EXECUTOR_TYPES['distributed']  # Add alias "dask" for distributed
@@ -361,3 +360,14 @@ def parsed_search_expressions(f):
 
     f = click.argument('expressions', callback=my_parse, nargs=-1)(f)
     return f
+
+
+def print_help_msg(command):
+    with click.Context(command) as ctx:
+        click.echo(command.get_help(ctx))
+
+
+def exit_on_empty_file(read_files_list):
+    if len(read_files_list) == 0:
+        click.echo("All files are empty, exit")
+        sys.exit(1)
