@@ -8,6 +8,7 @@ Functions for working with YAML documents and configurations
 import gzip
 import json
 import logging
+import math
 import sys
 import collections.abc
 from collections import OrderedDict
@@ -18,6 +19,7 @@ from urllib.parse import urlparse
 from urllib.request import urlopen
 from typing import Dict, Any, Mapping
 from copy import deepcopy
+from uuid import UUID
 
 import numpy
 import toolz  # type: ignore[import]
@@ -262,7 +264,14 @@ class NoDatesSafeLoader(SafeLoader):  # pylint: disable=too-many-ancestors
 NoDatesSafeLoader.remove_implicit_resolver('tag:yaml.org,2002:timestamp')
 
 
-class InvalidDocException(Exception):
+class InvalidDocException(Exception):  # noqa: N818
+    pass
+
+
+class UnknownMetadataType(InvalidDocException):
+    """
+    Specific exception to raise on a product with an unknown metadata type
+    """
     pass
 
 
@@ -284,6 +293,30 @@ def get_doc_offset_safe(offset, document, value_if_missing=None):
     return toolz.get_in(offset, document, default=value_if_missing)
 
 
+def documents_equal(d1, d2):
+    if d1.__class__ != d2.__class__:
+        return False
+    if isinstance(d1, str):
+        return d1 == d2
+    elif isinstance(d1, dict):
+        if set(d1.keys()) != set(d2.keys()):
+            return False
+        return all(documents_equal(d1[k], d2[k]) for k in d1)
+    elif isinstance(d1, list):
+        if len(d1) != len(d2):
+            return False
+        for i in range(len(d1)):
+            if not documents_equal(d1[i], d2[i]):
+                return False
+        return True
+    elif isinstance(d1, float):
+        if math.isnan(d1) and math.isnan(d2):
+            return True
+        return math.isclose(d1, d2, abs_tol=1e-10)
+    else:
+        return d1 == d2
+
+
 def transform_object_tree(f, o, key_transform=lambda k: k):
     """
     Apply a function (f) on all the values in the given document tree (o), returning a new document of
@@ -313,10 +346,15 @@ def transform_object_tree(f, o, key_transform=lambda k: k):
     return f(o)
 
 
-def metadata_subset(element, document) -> bool:
+def metadata_subset(element, document, full_recursion=False) -> bool:
     """
     Recursively check if one metadata document/object is a subset of another
 
+    full_recursion=False emulates the jsonb "contains" operator used by the postgis and postgres driver.
+        (This is used by the in-memory driver to implement search_by_metadata.)
+
+    full_recursion=True allows arbitrary depth matching.
+
     :param element: The document/object to search for
     :param document: The document/object to search in
     :return: True if element is a subset of document
@@ -324,30 +362,35 @@ def metadata_subset(element, document) -> bool:
     if isinstance(element, dict) and isinstance(document, dict):
         matches = True
         for k in element.keys():
-            if k not in document or not metadata_subset(element[k], document[k]):
+            if k not in document or not metadata_subset(element[k], document[k], full_recursion=full_recursion):
                 matches = False
                 break
         if matches:
             return True
+        if full_recursion:
+            for k in document.keys():
+                if metadata_subset(element, document[k], full_recursion=full_recursion):
+                    return True
+    elif isinstance(document, dict) and full_recursion:
         for k in document.keys():
-            if metadata_subset(element, document[k]):
-                return True
-    elif isinstance(document, dict):
-        for k in document.keys():
-            if metadata_subset(element, document[k]):
+            if metadata_subset(element, document[k], full_recursion=full_recursion):
                 return True
     elif isinstance(element, list) or isinstance(element, tuple):
         matches = True
         for i in element:
-            if not metadata_subset(i, document):
+            if not metadata_subset(i, document, full_recursion=full_recursion):
                 matches = False
                 break
         if matches:
             return True
     elif isinstance(document, list) or isinstance(document, tuple):
         for i in document:
-            if metadata_subset(element, i):
-                return True
+            if full_recursion:
+                if metadata_subset(element, i, full_recursion=full_recursion):
+                    return True
+            else:
+                if element == i:
+                    return True
     else:
         return element == document
     return False
@@ -372,6 +415,7 @@ class SimpleDocNav(object):
         self._doc_without = None
         self._sources_path = ('lineage', 'source_datasets')
         self._sources = None
+        self._doc_uuid = None
 
     @property
     def doc(self):
@@ -386,7 +430,11 @@ class SimpleDocNav(object):
 
     @property
     def id(self):
-        return self._doc.get('id', None)
+        if not self._doc_uuid:
+            doc_id = self._doc.get('id', None)
+            if doc_id:
+                self._doc_uuid = doc_id if isinstance(doc_id, UUID) else UUID(doc_id)
+        return self._doc_uuid
 
     @property
     def sources(self):
