@@ -70,126 +70,88 @@ _pseudo_telemetry_dataset_type = {
 }
 
 
-def test_archive_datasets(index, local_config, default_metadata_type):
-    dataset_type = index.products.add_document(_pseudo_telemetry_dataset_type)
-    with index.transaction() as transaction:
-        was_inserted = transaction._connection.insert_dataset(
-            _telemetry_dataset,
-            _telemetry_uuid,
-            dataset_type.id
-        )
-
-    assert was_inserted
-    assert index.datasets.has(_telemetry_uuid)
-
+def test_archive_datasets(index, local_config, ls8_eo3_dataset):
     datasets = index.datasets.search_eager()
     assert len(datasets) == 1
     assert datasets[0].is_active
 
-    index.datasets.archive([_telemetry_uuid])
+    index.datasets.archive([ls8_eo3_dataset.id])
     datasets = index.datasets.search_eager()
     assert len(datasets) == 0
 
     # The model should show it as archived now.
-    indexed_dataset = index.datasets.get(_telemetry_uuid)
+    indexed_dataset = index.datasets.get(ls8_eo3_dataset.id)
     assert indexed_dataset.is_archived
     assert not indexed_dataset.is_active
 
-    index.datasets.restore([_telemetry_uuid])
+    index.datasets.restore([ls8_eo3_dataset.id])
     datasets = index.datasets.search_eager()
     assert len(datasets) == 1
 
     # And now active
-    indexed_dataset = index.datasets.get(_telemetry_uuid)
+    indexed_dataset = index.datasets.get(ls8_eo3_dataset.id)
     assert indexed_dataset.is_active
     assert not indexed_dataset.is_archived
 
 
-def test_purge_datasets(index, local_config, default_metadata_type, clirunner):
-    # Create dataset
-    dataset_type = index.products.add_document(_pseudo_telemetry_dataset_type)
-    with index.transaction() as transaction:
-        was_inserted = transaction._connection.insert_dataset(
-            _telemetry_dataset,
-            _telemetry_uuid,
-            dataset_type.id
-        )
-
-    assert was_inserted
-    assert index.datasets.has(_telemetry_uuid)
+def test_purge_datasets(index, ls8_eo3_dataset):
+    assert index.datasets.has(ls8_eo3_dataset.id)
     datasets = index.datasets.search_eager()
     assert len(datasets) == 1
     assert datasets[0].is_active
 
     # Archive dataset
-    index.datasets.archive([_telemetry_uuid])
+    index.datasets.archive([ls8_eo3_dataset.id])
     datasets = index.datasets.search_eager()
     assert len(datasets) == 0
 
     # The model should show it as archived now.
-    indexed_dataset = index.datasets.get(_telemetry_uuid)
+    indexed_dataset = index.datasets.get(ls8_eo3_dataset.id)
     assert indexed_dataset.is_archived
     assert not indexed_dataset.is_active
 
     # Purge dataset
-    index.datasets.purge([_telemetry_uuid])
-    assert index.datasets.get(_telemetry_uuid) is None
+    index.datasets.purge([ls8_eo3_dataset.id])
+    assert index.datasets.get(ls8_eo3_dataset.id) is None
 
 
-def test_purge_datasets_cli(index, local_config, default_metadata_type, clirunner):
-    dataset_type = index.products.add_document(_pseudo_telemetry_dataset_type)
-
-    # Attempt to purge non-existent dataset should fail
-    clirunner(['dataset', 'purge', str(_telemetry_uuid)], expect_success=False)
-
-    # Create dataset
-    with index.transaction() as transaction:
-        was_inserted = transaction._connection.insert_dataset(
-            _telemetry_dataset,
-            _telemetry_uuid,
-            dataset_type.id
-        )
-    assert was_inserted
+def test_purge_datasets_cli(index, ls8_eo3_dataset, clirunner):
+    dsid = ls8_eo3_dataset.id
 
     # Attempt to purge non-archived dataset should fail
-    clirunner(['dataset', 'purge', str(_telemetry_uuid)], expect_success=False)
+    clirunner(['dataset', 'purge', str(dsid)], expect_success=False)
 
     # Archive dataset
-    index.datasets.archive([_telemetry_uuid])
-    indexed_dataset = index.datasets.get(_telemetry_uuid)
+    index.datasets.archive([dsid])
+    indexed_dataset = index.datasets.get(dsid)
     assert indexed_dataset.is_archived
 
     # Test CLI dry run
-    clirunner(['dataset', 'purge', '--dry-run', str(_telemetry_uuid)])
-    indexed_dataset = index.datasets.get(_telemetry_uuid)
+    clirunner(['dataset', 'purge', '--dry-run', str(dsid)])
+    indexed_dataset = index.datasets.get(dsid)
     assert indexed_dataset.is_archived
 
     # Test CLI purge
-    clirunner(['dataset', 'purge', str(_telemetry_uuid)])
-    assert index.datasets.get(_telemetry_uuid) is None
+    clirunner(['dataset', 'purge', str(dsid)])
+    assert index.datasets.get(dsid) is None
+
+    # Attempt to purge non-existent dataset should fail
+    clirunner(['dataset', 'purge', str(dsid)], expect_success=False)
 
 
-def test_purge_all_datasets_cli(index, local_config, default_metadata_type, clirunner):
-    dataset_type = index.products.add_document(_pseudo_telemetry_dataset_type)
-
-    # Create dataset
-    with index.transaction() as transaction:
-        was_inserted = transaction._connection.insert_dataset(
-            _telemetry_dataset,
-            _telemetry_uuid,
-            dataset_type.id
-        )
-    assert was_inserted
+def test_purge_all_datasets_cli(index, local_config, ls8_eo3_dataset, clirunner):
+    product = ls8_eo3_dataset.product.id
+    dsid = ls8_eo3_dataset.id
 
     # archive all datasets
     clirunner(['dataset', 'archive', '--all'])
 
-    indexed_dataset = index.datasets.get(_telemetry_uuid)
+    indexed_dataset = index.datasets.get(dsid)
     assert indexed_dataset.is_archived
 
     # Restore all datasets
     clirunner(['dataset', 'restore', '--all'])
-    indexed_dataset = index.datasets.get(_telemetry_uuid)
+    indexed_dataset = index.datasets.get(dsid)
     assert not indexed_dataset.is_archived
 
     # Archive again
@@ -197,76 +159,47 @@ def test_purge_all_datasets_cli(index, local_config, default_metadata_type, clir
 
     # and purge
     clirunner(['dataset', 'purge', '--all'])
-    assert index.datasets.get(_telemetry_uuid) is None
-
-
-@pytest.fixture
-def telemetry_dataset(index: Index, default_metadata_type) -> Dataset:
-    dataset_type = index.products.add_document(_pseudo_telemetry_dataset_type)
-    assert not index.datasets.has(_telemetry_uuid)
-
-    with index.transaction() as transaction:
-        was_inserted = transaction._connection.insert_dataset(
-            _telemetry_dataset,
-            _telemetry_uuid,
-            dataset_type.id
-        )
-    assert was_inserted
-
-    return index.datasets.get(_telemetry_uuid)
+    assert index.datasets.get(dsid) is None
 
 
 def test_index_duplicate_dataset(index: Index,
                                  local_config,
-                                 default_metadata_type) -> None:
-    dataset_type = index.products.add_document(_pseudo_telemetry_dataset_type)
-    assert not index.datasets.has(_telemetry_uuid)
-
-    with index.transaction() as transaction:
-        was_inserted = transaction._connection.insert_dataset(
-            _telemetry_dataset,
-            _telemetry_uuid,
-            dataset_type.id
-        )
-
-    assert was_inserted
-    assert index.datasets.has(_telemetry_uuid)
+                                 ls8_eo3_dataset) -> None:
+    product = ls8_eo3_dataset.product
+    dsid = ls8_eo3_dataset.id
+    assert index.datasets.has(dsid)
 
     # Insert again.
-    with index._db._connect() as connection:
-        was_inserted = connection.insert_dataset(
-            _telemetry_dataset,
-            _telemetry_uuid,
-            dataset_type.id
-        )
-        assert was_inserted is False
+    ds = Dataset(product, ls8_eo3_dataset.metadata_doc,
+                 uris=ls8_eo3_dataset.uris)
+    index.datasets.add(ds, with_lineage=False)
 
-    assert index.datasets.has(_telemetry_uuid)
+    assert index.datasets.has(dsid)
 
 
-def test_has_dataset(index: Index, telemetry_dataset: Dataset) -> None:
-    assert index.datasets.has(_telemetry_uuid)
-    assert index.datasets.has(str(_telemetry_uuid))
+def test_has_dataset(index: Index, ls8_eo3_dataset: Dataset) -> None:
+    assert index.datasets.has(ls8_eo3_dataset.id)
+    assert index.datasets.has(str(ls8_eo3_dataset.id))
 
     assert not index.datasets.has(UUID('f226a278-e422-11e6-b501-185e0f80a5c0'))
     assert not index.datasets.has('f226a278-e422-11e6-b501-185e0f80a5c0')
 
-    assert index.datasets.bulk_has([_telemetry_uuid, UUID('f226a278-e422-11e6-b501-185e0f80a5c0')]) == [True, False]
-    assert index.datasets.bulk_has([str(_telemetry_uuid), 'f226a278-e422-11e6-b501-185e0f80a5c0']) == [True, False]
+    assert index.datasets.bulk_has([ls8_eo3_dataset.id, UUID('f226a278-e422-11e6-b501-185e0f80a5c0')]) == [True, False]
+    assert index.datasets.bulk_has([str(ls8_eo3_dataset.id), 'f226a278-e422-11e6-b501-185e0f80a5c0']) == [True, False]
 
 
-def test_get_dataset(index: Index, telemetry_dataset: Dataset) -> None:
-    assert index.datasets.has(_telemetry_uuid)
-    assert index.datasets.has(str(_telemetry_uuid))
+def test_get_dataset(index: Index, ls8_eo3_dataset: Dataset) -> None:
+    assert index.datasets.has(ls8_eo3_dataset.id)
+    assert index.datasets.has(str(ls8_eo3_dataset.id))
 
-    assert index.datasets.bulk_has([_telemetry_uuid, 'f226a278-e422-11e6-b501-185e0f80a5c0']) == [True, False]
+    assert index.datasets.bulk_has([ls8_eo3_dataset.id, 'f226a278-e422-11e6-b501-185e0f80a5c0']) == [True, False]
 
     for tr in (lambda x: x, lambda x: str(x)):
-        ds = index.datasets.get(tr(_telemetry_uuid))
-        assert ds.id == _telemetry_uuid
+        ds = index.datasets.get(tr(ls8_eo3_dataset.id))
+        assert ds.id == ls8_eo3_dataset.id
 
-        ds, = index.datasets.bulk_get([tr(_telemetry_uuid)])
-        assert ds.id == _telemetry_uuid
+        ds, = index.datasets.bulk_get([tr(ls8_eo3_dataset.id)])
+        assert ds.id == ls8_eo3_dataset.id
 
     assert index.datasets.bulk_get(['f226a278-e422-11e6-b501-185e0f80a5c0',
                                     'f226a278-e422-11e6-b501-185e0f80a5c1']) == []
@@ -284,24 +217,57 @@ def test_transactions_api_ctx_mgr(index,
     with pytest.raises(Exception) as e:
         with index.transaction() as trans:
             assert index.datasets.get(ds1.id) is None
-            index.datasets.add(ds1)
+            index.datasets.add(ds1, with_lineage=False)
             assert index.datasets.get(ds1.id) is not None
             raise Exception("Rollback!")
     assert "Rollback!" in str(e.value)
     assert index.datasets.get(ds1.id) is None
     with index.transaction() as trans:
         assert index.datasets.get(ds1.id) is None
-        index.datasets.add(ds1)
+        index.datasets.add(ds1, with_lineage=False)
         assert index.datasets.get(ds1.id) is not None
     assert index.datasets.get(ds1.id) is not None
     with index.transaction() as trans:
-        index.datasets.add(ds2)
+        index.datasets.add(ds2, with_lineage=False)
         assert index.datasets.get(ds2.id) is not None
         raise trans.rollback_exception("Rollback")
     assert index.datasets.get(ds1.id) is not None
     assert index.datasets.get(ds2.id) is None
 
 
+def test_transactions_api_ctx_mgr_nested(index,
+                                         extended_eo3_metadata_type_doc,
+                                         ls8_eo3_product,
+                                         eo3_ls8_dataset_doc,
+                                         eo3_ls8_dataset2_doc):
+    from datacube.index.hl import Doc2Dataset
+    resolver = Doc2Dataset(index, products=[ls8_eo3_product.name], verify_lineage=False)
+    ds1, err = resolver(*eo3_ls8_dataset_doc)
+    ds2, err = resolver(*eo3_ls8_dataset2_doc)
+    with pytest.raises(Exception) as e:
+        with index.transaction() as trans_outer:
+            with index.transaction() as trans:
+                assert index.datasets.get(ds1.id) is None
+                index.datasets.add(ds1, False)
+                assert index.datasets.get(ds1.id) is not None
+                raise Exception("Rollback!")
+    assert "Rollback!" in str(e.value)
+    assert index.datasets.get(ds1.id) is None
+    with index.transaction() as trans_outer:
+        with index.transaction() as trans:
+            assert index.datasets.get(ds1.id) is None
+            index.datasets.add(ds1, False)
+            assert index.datasets.get(ds1.id) is not None
+    assert index.datasets.get(ds1.id) is not None
+    with index.transaction() as trans_outer:
+        with index.transaction() as trans:
+            index.datasets.add(ds2, False)
+            assert index.datasets.get(ds2.id) is not None
+            raise trans.rollback_exception("Rollback")
+    assert index.datasets.get(ds1.id) is not None
+    assert index.datasets.get(ds2.id) is None
+
+
 def test_transactions_api_manual(index,
                                  extended_eo3_metadata_type_doc,
                                  ls8_eo3_product,
@@ -312,17 +278,17 @@ def test_transactions_api_manual(index,
     ds1, err = resolver(*eo3_ls8_dataset_doc)
     ds2, err = resolver(*eo3_ls8_dataset2_doc)
     trans = index.transaction()
-    index.datasets.add(ds1)
+    index.datasets.add(ds1, False)
     assert index.datasets.get(ds1.id) is not None
     trans.begin()
-    index.datasets.add(ds2)
+    index.datasets.add(ds2, False)
     assert index.datasets.get(ds1.id) is not None
     assert index.datasets.get(ds2.id) is not None
     trans.rollback()
     assert index.datasets.get(ds1.id) is not None
     assert index.datasets.get(ds2.id) is None
     trans.begin()
-    index.datasets.add(ds2)
+    index.datasets.add(ds2, False)
     trans.commit()
     assert index.datasets.get(ds1.id) is not None
     assert index.datasets.get(ds2.id) is not None
@@ -339,18 +305,18 @@ def test_transactions_api_hybrid(index,
     ds2, err = resolver(*eo3_ls8_dataset2_doc)
     with index.transaction() as trans:
         assert index.datasets.get(ds1.id) is None
-        index.datasets.add(ds1)
+        index.datasets.add(ds1, False)
         assert index.datasets.get(ds1.id) is not None
         trans.rollback()
         assert index.datasets.get(ds1.id) is None
         trans.begin()
         assert index.datasets.get(ds1.id) is None
-        index.datasets.add(ds1)
+        index.datasets.add(ds1, False)
         assert index.datasets.get(ds1.id) is not None
         trans.commit()
         assert index.datasets.get(ds1.id) is not None
         trans.begin()
-        index.datasets.add(ds2)
+        index.datasets.add(ds2, False)
         assert index.datasets.get(ds2.id) is not None
         trans.rollback()
     assert index.datasets.get(ds1.id) is not None
@@ -410,14 +376,14 @@ def test_index_dataset_with_location(index: Index, default_metadata_type: Metada
     first_file = Path('/tmp/first/something.yaml').absolute()
     second_file = Path('/tmp/second/something.yaml').absolute()
 
-    type_ = index.products.add_document(_pseudo_telemetry_dataset_type)
-    dataset = Dataset(type_, _telemetry_dataset, uris=[first_file.as_uri()], sources={})
+    product = index.products.add_document(_pseudo_telemetry_dataset_type)
+    dataset = Dataset(product, _telemetry_dataset, uris=[first_file.as_uri()], sources={})
     index.datasets.add(dataset)
     stored = index.datasets.get(dataset.id)
 
     assert stored.id == _telemetry_uuid
     # TODO: Dataset types?
-    assert stored.type.id == type_.id
+    assert stored.product.id == product.id
     assert stored.metadata_type.id == default_metadata_type.id
     assert stored.local_path == Path(first_file)
 
@@ -515,7 +481,7 @@ def test_index_dataset_with_location(index: Index, default_metadata_type: Metada
     # Check order of uris is preserved when indexing with more than one
     second_ds_doc = copy.deepcopy(_telemetry_dataset)
     second_ds_doc['id'] = '366f32d8-e1f8-11e6-94b4-185e0f80a589'
-    index.datasets.add(Dataset(type_, second_ds_doc, uris=['file:///a', 'file:///b'], sources={}))
+    index.datasets.add(Dataset(product, second_ds_doc, uris=['file:///a', 'file:///b'], sources={}))
 
     # test order using get_locations function
     assert index.datasets.get_locations(second_ds_doc['id']) == ['file:///a', 'file:///b']
@@ -524,7 +490,7 @@ def test_index_dataset_with_location(index: Index, default_metadata_type: Metada
     assert index.datasets.get(second_ds_doc['id']).uris == ['file:///a', 'file:///b']
 
     # test update, this should prepend file:///c, file:///d to the existing list
-    index.datasets.update(Dataset(type_, second_ds_doc, uris=['file:///a', 'file:///c', 'file:///d'], sources={}))
+    index.datasets.update(Dataset(product, second_ds_doc, uris=['file:///a', 'file:///c', 'file:///d'], sources={}))
     assert index.datasets.get_locations(second_ds_doc['id']) == ['file:///c', 'file:///d', 'file:///a', 'file:///b']
     assert index.datasets.get(second_ds_doc['id']).uris == ['file:///c', 'file:///d', 'file:///a', 'file:///b']
 
@@ -532,7 +498,7 @@ def test_index_dataset_with_location(index: Index, default_metadata_type: Metada
     # Add a second dataset with a different location (to catch lack of joins, filtering etc)
     second_ds_doc = copy.deepcopy(_telemetry_dataset)
     second_ds_doc['id'] = '366f32d8-e1f8-11e6-94b4-185e0f80a5c0'
-    index.datasets.add(Dataset(type_, second_ds_doc, uris=[second_file.as_uri()], sources={}))
+    index.datasets.add(Dataset(product, second_ds_doc, uris=[second_file.as_uri()], sources={}))
     for mode in ('exact', 'prefix', None):
         dataset_ids = [d.id for d in index.datasets.get_datasets_for_location(first_file.as_uri(), mode=mode)]
         assert dataset_ids == [dataset.id]
@@ -546,3 +512,14 @@ def test_index_dataset_with_location(index: Index, default_metadata_type: Metada
 def utc_now():
     # utcnow() doesn't include a tzinfo.
     return datetime.datetime.utcnow().replace(tzinfo=tz.tzutc())
+
+
+def test_bulk_reads_transaction(index, extended_eo3_metadata_type_doc,
+                                ls8_eo3_product,
+                                eo3_ls8_dataset_doc,
+                                eo3_ls8_dataset2_doc
+                                ):
+    with pytest.raises(ValueError) as e:
+        with index.datasets._db_connection() as conn:
+            conn.bulk_simple_dataset_search(batch_size=2)
+    assert "within a transaction" in str(e.value)
