@@ -16,43 +16,60 @@ import pytest
 EXAMPLE_DATASET_TYPE_DOCS = map(str, Path(__file__).parent.parent.
                                 joinpath('docs', 'config_samples', 'dataset_types').glob('**/*.yaml'))
 
+
 # Documents that shouldn't be accepted as mapping docs.
 INVALID_MAPPING_DOCS = map(str, Path(__file__).parent.parent.joinpath('docs').glob('*'))
 
 
-def _dataset_type_count(db):
-    with db.connect() as connection:
+def _dataset_type_count(index):
+    with index._active_connection() as connection:
         return len(list(connection.get_all_products()))
 
 
-def test_add_example_dataset_types(clirunner, initialised_postgres_db, default_metadata_type):
+def test_add_example_dataset_types(clirunner, index, default_metadata_type, eo3_product_paths, ext_eo3_mdt_path):
     """
     Add example mapping docs, to ensure they're valid and up-to-date.
 
     We add them all to a single database to check for things like duplicate ids.
-
-    :type initialised_postgres_db: datacube.drivers.postgres._connections.PostgresDb
     """
-    existing_mappings = _dataset_type_count(initialised_postgres_db)
+    existing_mappings = _dataset_type_count(index)
 
-    print('{} mappings'.format(existing_mappings))
-    for mapping_path in EXAMPLE_DATASET_TYPE_DOCS:
-        print('Adding mapping {}'.format(mapping_path))
+    if index.supports_legacy:
+        # Legacy EO test examples
+        print('{} mappings'.format(existing_mappings))
+        for mapping_path in EXAMPLE_DATASET_TYPE_DOCS:
+            print('Adding mapping {}'.format(mapping_path))
 
-        result = clirunner(['-v', 'product', 'add', mapping_path])
+            result = clirunner(['-v', 'product', 'add', mapping_path])
+            assert result.exit_code == 0
+
+            mappings_count = _dataset_type_count(index)
+            assert mappings_count > existing_mappings, "Mapping document was not added: " + str(mapping_path)
+            existing_mappings = mappings_count
+
+        result = clirunner(['-v', 'metadata', 'show', '-f', 'json', 'eo'],
+                           expect_success=True)
         assert result.exit_code == 0
 
-        mappings_count = _dataset_type_count(initialised_postgres_db)
-        assert mappings_count > existing_mappings, "Mapping document was not added: " + str(mapping_path)
-        existing_mappings = mappings_count
-
-    result = clirunner(['-v', 'metadata', 'list'])
+    # EO3 test examples
+    result = clirunner(['-v', 'metadata', 'add', ext_eo3_mdt_path])
     assert result.exit_code == 0
 
-    result = clirunner(['-v', 'metadata', 'show', '-f', 'json', 'eo'],
+    for path in eo3_product_paths:
+        result = clirunner(['-v', 'product', 'add', path])
+        assert result.exit_code == 0
+
+        mappings_count = _dataset_type_count(index)
+        assert mappings_count > existing_mappings, "Mapping document was not added: " + str(path)
+        existing_mappings = mappings_count
+
+    result = clirunner(['-v', 'metadata', 'show', '-f', 'json', 'eo3'],
                        expect_success=True)
     assert result.exit_code == 0
 
+    result = clirunner(['-v', 'metadata', 'list'])
+    assert result.exit_code == 0
+
     result = clirunner(['-v', 'metadata', 'show'],
                        expect_success=True)
     assert result.exit_code == 0
@@ -70,20 +87,17 @@ def test_add_example_dataset_types(clirunner, initialised_postgres_db, default_m
                            expect_success=False)
         assert result.exit_code == 1
 
-        result = clirunner(['-v', 'product', 'show', '-f', 'json', 'ls8_level1_usgs'],
+        result = clirunner(['-v', 'product', 'show', '-f', 'json', 'ga_ls8c_ard_3'],
                            expect_success=False)
         assert result.exit_code == 0
 
-        result = clirunner(['-v', 'product', 'show', '-f', 'yaml', 'ls8_level1_usgs'],
+        result = clirunner(['-v', 'product', 'show', '-f', 'yaml', 'ga_ls8c_ard_3'],
                            expect_success=False)
         assert result.exit_code == 0
 
 
-def test_error_returned_on_invalid(clirunner, initialised_postgres_db):
-    """
-    :type initialised_postgres_db: datacube.drivers.postgres._connections.PostgresDb
-    """
-    assert _dataset_type_count(initialised_postgres_db) == 0
+def test_error_returned_on_invalid(clirunner, index):
+    assert _dataset_type_count(index) == 0
 
     for mapping_path in INVALID_MAPPING_DOCS:
         result = clirunner(
@@ -95,10 +109,10 @@ def test_error_returned_on_invalid(clirunner, initialised_postgres_db):
             expect_success=False
         )
         assert result.exit_code != 0, "Success return code for invalid document."
-        assert _dataset_type_count(initialised_postgres_db) == 0, "Invalid document was added to DB"
+        assert _dataset_type_count(index) == 0, "Invalid document was added to DB"
 
 
-def test_config_check(clirunner, initialised_postgres_db, local_config):
+def test_config_check(clirunner, index, local_config):
     """
     :type local_config: datacube.config.LocalConfig
     """
@@ -119,7 +133,7 @@ def test_config_check(clirunner, initialised_postgres_db, local_config):
     assert user_regex.match(result.output)
 
 
-def test_list_users_does_not_fail(clirunner, local_config, initialised_postgres_db):
+def test_list_users_does_not_fail(clirunner, local_config, index):
     """
     :type local_config: datacube.config.LocalConfig
     """
@@ -134,7 +148,7 @@ def test_list_users_does_not_fail(clirunner, local_config, initialised_postgres_
     assert result.exit_code == 0
 
 
-def test_db_init_noop(clirunner, local_config, ls5_telem_type):
+def test_db_init_noop(clirunner, local_config, ls8_eo3_product):
     # Run on an existing database.
     result = clirunner(
         [
@@ -143,12 +157,13 @@ def test_db_init_noop(clirunner, local_config, ls5_telem_type):
     )
     assert 'Updated.' in result.output
     # It should not rebuild indexes by default
-    assert 'Dropping index: dix_{}'.format(ls5_telem_type.name) not in result.output
+    assert 'Dropping index: dix_{}'.format(ls8_eo3_product.name) not in result.output
 
     result = clirunner(['metadata', 'list'])
     assert "eo3 " in result.output
 
 
+@pytest.mark.parametrize('datacube_env_name', ('datacube', ))
 def test_db_init_rebuild(clirunner, local_config, ls5_telem_type):
     if local_config._env == "datacube":
         from datacube.drivers.postgres import _dynamic
@@ -177,31 +192,31 @@ def test_db_init_rebuild(clirunner, local_config, ls5_telem_type):
     ) in result.output
 
 
-def test_db_init(clirunner, initialised_postgres_db):
-    if initialised_postgres_db.driver_name == "postgis":
+def test_db_init(clirunner, index):
+    if index._db.driver_name == "postgis":
         from datacube.drivers.postgis._core import drop_db, has_schema
     else:
         from datacube.drivers.postgres._core import drop_db, has_schema
 
-    with initialised_postgres_db.connect() as connection:
+    with index._db._connect() as connection:
         drop_db(connection._connection)
 
-        assert not has_schema(initialised_postgres_db._engine, connection._connection)
+        assert not has_schema(index._db._engine, connection._connection)
 
     # Run on an empty database.
-    if initialised_postgres_db.driver_name == "postgis":
+    if index._db.driver_name == "postgis":
         result = clirunner(['-E', 'experimental', 'system', 'init'])
     else:
         result = clirunner(['system', 'init'])
 
     assert 'Created.' in result.output
 
-    with initialised_postgres_db.connect() as connection:
-        assert has_schema(initialised_postgres_db._engine, connection._connection)
+    with index._db._connect() as connection:
+        assert has_schema(index._db._engine, connection._connection)
 
 
-def test_add_no_such_product(clirunner, initialised_postgres_db):
-    result = clirunner(['dataset', 'add', '--dtype', 'no_such_product'], expect_success=False)
+def test_add_no_such_product(clirunner, index):
+    result = clirunner(['dataset', 'add', '--dtype', 'no_such_product', '/tmp'], expect_success=False)
     assert result.exit_code != 0
     assert "DEPRECATED option detected" in result.output
     assert "ERROR Supplied product name" in result.output
@@ -213,13 +228,13 @@ def test_add_no_such_product(clirunner, initialised_postgres_db):
     # Test that names are escaped
     ('test_user_"invalid+_chars_{n}', None),
     ('test_user_invalid_desc_{n}', 'Invalid "\' chars in description')])
-def example_user(clirunner, initialised_postgres_db, request):
+def example_user(clirunner, index, request):
     username, description = request.param
 
     username = username.format(n=random.randint(111111, 999999))
 
     # test_roles = (user_name for role_name, user_name, desc in roles if user_name.startswith('test_'))
-    with initialised_postgres_db.connect() as connection:
+    with index._db._connect() as connection:
         users = (user_name for role_name, user_name, desc in connection.list_users())
         if username in users:
             connection.drop_users([username])
@@ -229,7 +244,7 @@ def example_user(clirunner, initialised_postgres_db, request):
 
     yield username, description
 
-    with initialised_postgres_db.connect() as connection:
+    with index._db._connect() as connection:
         users = (user_name for role_name, user_name, desc in connection.list_users())
         if username in users:
             connection.drop_users([username])
