@@ -69,7 +69,7 @@ def test_new_xr_load(data_folder):
     sources = Datacube.group_datasets([ds], 'time')
 
     im, meta = rio_slurp(str(data_folder) + '/test.tif')
-    measurements = [ds.type.measurements[n] for n in ('a', 'b')]
+    measurements = [ds.product.measurements[n] for n in ('a', 'b')]
     measurements[1]['fuser'] = lambda dst, src: _default_fuser(dst, src, measurements[1].nodata)
 
     xx, _ = xr_load(sources, meta.gbox, measurements, rdr)
