@@ -12,6 +12,7 @@ from pathlib import Path
 from collections import OrderedDict
 from types import SimpleNamespace
 from typing import Tuple, Iterable
+from uuid import UUID, uuid4
 
 import numpy as np
 import pytest
@@ -55,7 +56,23 @@ doc_changes = [
     ({'a': 1}, {'b': 1}, [(('a',), 1, MISSING), (('b',), MISSING, 1)]),
     ({'a': {'b': 1}}, {'a': {'b': 2}}, [(('a', 'b'), 1, 2)]),
     ({}, {'b': 1}, [(('b',), MISSING, 1)]),
-    ({'a': {'c': 1}}, {'a': {'b': 1}}, [(('a', 'b'), MISSING, 1), (('a', 'c'), 1, MISSING)])
+    ({'a': {'c': 1}}, {'a': {'b': 1}}, [(('a', 'b'), MISSING, 1), (('a', 'c'), 1, MISSING)]),
+    # Test tuple vs list, for geometry coordinates handling and expect no changes
+    (
+        [[635235.0, -2930535.0], [635235.0, -2930535.0]],
+        ((635235.0, -2930535.0), (635235.0, -2930535.0),),
+        []
+    ),
+    (
+        [[[635235.0, -2930535.0], [635235.0, -2930535.0]]],
+        (((635235.0, -2930535.0), (635235.0, -2930535.0)),),
+        []
+    ),
+    (
+        {'coordinates': [[[635235.0, -2930535.0], [635235.0, -2930535.0]]]},
+        {'coordinates': (((635235.0, -2930535.0), (635235.0, -2930535.0)),)},
+        []
+    )
 ]
 
 
@@ -221,7 +238,7 @@ def test_dataset_maker():
 
     assert a.id != b.id
     assert a.doc['creation_dt'] == b.doc['creation_dt']
-    assert isinstance(a.id, str)
+    assert isinstance(a.id, UUID)
     assert a.sources == {}
 
     a1, a2 = [dataset_maker(i)('A', product_type='eo') for i in (0, 1)]
@@ -305,10 +322,13 @@ def test_simple_doc_nav():
       +--> E
     """
 
-    def node(name, **kwargs):
-        return dict(id=name, lineage=dict(source_datasets=kwargs))
+    nu_map = {n: uuid4() for n in ['A', 'B', 'C', 'D', 'E']}
+    un_map = {u: n for n, u in nu_map.items()}
 
-    A, _, C, _, _ = make_graph_abcde(node)
+    def node(name, **kwargs):
+        return dict(id=nu_map[name], lineage=dict(source_datasets=kwargs))
+
+    A, _, C, _, _ = make_graph_abcde(node)  # noqa: N806
     rdr = SimpleDocNav(A)
 
     assert rdr.doc == A
@@ -320,7 +340,7 @@ def test_simple_doc_nav():
     assert isinstance(rdr.sources_path, tuple)
 
     def visitor(node, name=None, depth=0, out=None):
-        s = '{}:{}:{:d}'.format(node.id, name if name else '..', depth)
+        s = '{}:{}:{:d}'.format(un_map[node.id], name if name else '..', depth)
         out.append(s)
 
     expect_preorder = '''
@@ -351,17 +371,17 @@ A:..:0
 
     fv = flatten_datasets(rdr)
 
-    assert len(fv['A']) == 1
-    assert len(fv['C']) == 2
-    assert len(fv['E']) == 1
-    assert set(fv.keys()) == set('ABCDE')
+    assert len(fv[nu_map['A']]) == 1
+    assert len(fv[nu_map['C']]) == 2
+    assert len(fv[nu_map['E']]) == 1
+    assert set(fv.keys()) == set(un_map.keys())
 
     fv, dg = flatten_datasets(rdr, with_depth_grouping=True)
 
-    assert len(fv['A']) == 1
-    assert len(fv['C']) == 2
-    assert len(fv['E']) == 1
-    assert set(fv.keys()) == set('ABCDE')
+    assert len(fv[nu_map['A']]) == 1
+    assert len(fv[nu_map['C']]) == 2
+    assert len(fv[nu_map['E']]) == 1
+    assert set(fv.keys()) == set(un_map.keys())
     assert isinstance(dg, list)
     assert len(dg) == 4
     assert [len(dss) for dss in dg] == [1, 3, 2, 1]
@@ -369,10 +389,9 @@ A:..:0
     def to_set(xx):
         return set(x.id for x in xx)
 
-    assert [set(s) for s in ('A',
-                             'BCE',
-                             'CD',
-                             'D')] == [to_set(xx) for xx in dg]
+    assert [set(nu_map[n] for n in s)
+            for s in ('A', 'BCE', 'CD', 'D')
+            ] == [to_set(xx) for xx in dg]
 
     with pytest.raises(ValueError):
         SimpleDocNav([])
@@ -509,11 +528,10 @@ def test_jsonify():
                                     'c': datetime(2016, 3, 11),
                                     'd': np.dtype('int16'),
                                     }).items()) == [
-                                    ('a', (1.0, 2.0, 3.0)),
-                                    ('b', 'Infinity'),
-                                    ('c', '2016-03-11T00:00:00'),
-                                    ('d', 'int16'),
-                                    ]
+                                        ('a', (1.0, 2.0, 3.0)),
+                                        ('b', 'Infinity'),
+                                        ('c', '2016-03-11T00:00:00'),
+                                        ('d', 'int16'), ]
 
     # Converts keys to strings:
     assert sorted(jsonify_document({1: 'a', '2': Decimal('2')}).items()) == [
@@ -595,23 +613,84 @@ def test_transform_object_tree():
 
 
 def test_document_subset():
+    # metadata_subset should emulate behaviour of postgres "contains" (@>) operator.
     assert metadata_subset(37, 37)
-    assert metadata_subset(37, {"b": 37})
-    assert metadata_subset(37, {"b": "foo", "a": {"d": 37}})
-    assert metadata_subset(37, {"b": [56, 36, 37]})
+    assert metadata_subset(37, [1, 111, 37])
+    assert metadata_subset(37, [1, 37, 444])
+    assert not metadata_subset(37, {"b": 37})
+    assert not metadata_subset(37, {"b": "foo", "a": {"d": 37}})
+    assert not metadata_subset(37, {"b": [56, 36, 37]})
     assert not metadata_subset(37, {"b": [56, 36, 57]})
 
     assert metadata_subset({"a": "foo"}, {"b": "blob", "a": "foo", "d": [76, 345, 34, 54]})
-    assert metadata_subset({"a": "foo"}, {"b": "blob", "d": [{"b": "glue", "a": "nope"}, {"a": "foo"}, 54]})
-    assert metadata_subset({"a": "foo"}, {"g": "goo", "h": {"f": "foo", "j": {"w": "who", "a": "foo"}}})
-    assert metadata_subset(
+    assert not metadata_subset({"a": "foo"}, {"b": "blob", "d": [{"b": "glue", "a": "nope"}, {"a": "foo"}, 54]})
+    assert not metadata_subset({"a": "foo"}, {"g": "goo", "h": {"f": "foo", "j": {"w": "who", "a": "foo"}}})
+    assert not metadata_subset(
         {"a": "foo", "k": {"b": [34, 11]}},
         {"g": "goo", "h": {"a": "foo", "k": {"b": [11, 234, 34, 35]}, "d": "doop"}}
     )
+    assert metadata_subset(
+        {"a": "foo", "k": {"b": [34, 11]}},
+        {"a": "foo", "k": {"b": [11, 234, 34, 35]}, "d": "doop"}
+    )
     assert not metadata_subset(
         {"a": "foo", "k": {"b": [34, 11]}},
         {"g": "goo", "h": {"a": "foo", "b": [11, 34], "k": {"b": [11, 234, 35]}, "d": "doop"}}
     )
+    assert not metadata_subset(
+        {"a": "foo", "k": {"b": [34, 11]}},
+        {"a": "foo", "b": [11, 34], "k": {"b": [11, 234, 35]}, "d": "doop"}
+    )
 
     assert metadata_subset([35, 47, 58], [0, 35, 47, 58, 102])
-    assert metadata_subset([35, 47, 58], {"a": "foo", "b": [35, 47, 52, 58]})
+    assert not metadata_subset([35, 47, 58], {"a": "foo", "b": [35, 47, 52, 58]})
+
+
+def test_document_subset_full_recursion():
+    # metadata_subset should emulate behaviour of postgres "contains" (@>) operator.
+    assert metadata_subset(37, 37, full_recursion=True)
+    assert metadata_subset(37, [1, 111, 37], full_recursion=True)
+    assert metadata_subset(37, [1, 37, 444], full_recursion=True)
+    assert metadata_subset(37, {"b": 37}, full_recursion=True)
+    assert metadata_subset(37, {"b": "foo", "a": {"d": 37}}, full_recursion=True)
+    assert metadata_subset(37, {"b": [56, 36, 37]}, full_recursion=True)
+    assert not metadata_subset(37, {"b": [56, 36, 57]}, full_recursion=True)
+
+    assert metadata_subset({"a": "foo"}, {"b": "blob", "a": "foo", "d": [76, 345, 34, 54]}, full_recursion=True)
+    assert metadata_subset({"a": "foo"}, {"b": "blob", "d": [{"b": "glue", "a": "nope"}, {"a": "foo"}, 54]},
+                           full_recursion=True)
+    assert metadata_subset({"a": "foo"}, {"g": "goo", "h": {"f": "foo", "j": {"w": "who", "a": "foo"}}},
+                           full_recursion=True)
+    assert metadata_subset(
+        {"a": "foo", "k": {"b": [34, 11]}},
+        {"g": "goo", "h": {"a": "foo", "k": {"b": [11, 234, 34, 35]}, "d": "doop"}},
+        full_recursion=True
+    )
+    assert metadata_subset(
+        {"a": "foo", "k": {"b": [34, 11]}},
+        {"a": "foo", "k": {"b": [11, 234, 34, 35]}, "d": "doop"},
+        full_recursion=True
+    )
+    assert not metadata_subset(
+        {"a": "foo", "k": {"b": [34, 11]}},
+        {"g": "goo", "h": {"a": "foo", "b": [11, 34], "k": {"b": [11, 234, 35]}, "d": "doop"}},
+        full_recursion=True
+    )
+    assert not metadata_subset(
+        {"a": "foo", "k": {"b": [34, 11]}},
+        {"a": "foo", "b": [11, 34], "k": {"b": [11, 234, 35]}, "d": "doop"},
+        full_recursion=True
+    )
+
+    assert metadata_subset([35, 47, 58], [0, 35, 47, 58, 102], full_recursion=True)
+    assert metadata_subset([35, 47, 58], {"a": "foo", "b": [35, 47, 52, 58]}, full_recursion=True)
+
+
+def test_documents_equal():
+    from datacube.utils.documents import documents_equal
+    assert not documents_equal(7, "seven")
+    assert not documents_equal({"a": "ok", "b": "mediocre"}, {"c": "great", "b": "mediocre", "a": "ok"})
+    assert not documents_equal([0, 1, 2], [0, 1, 2, 3])
+    assert not documents_equal([0, 1, 2], [0, 2, 1])
+    assert not documents_equal(0.5, 0.50001)
+    assert documents_equal(0.5, 0.50000000001)
